@@ -19,6 +19,7 @@
 from contextlib import contextmanager
 import copy
 import datetime
+import io
 import logging
 import os
 import platform
@@ -28,11 +29,15 @@ try:
     import syslog
 except ImportError:
     syslog = None
+try:
+    from systemd import journal
+except ImportError:
+    journal = None
 import tempfile
 import time
+from unittest import mock
 
 from dateutil import tz
-import mock
 from oslo_config import cfg
 from oslo_config import fixture as fixture_config  # noqa
 from oslo_context import context
@@ -40,13 +45,13 @@ from oslo_context import fixture as fixture_context
 from oslo_i18n import fixture as fixture_trans
 from oslo_serialization import jsonutils
 from oslotest import base as test_base
-import six
 import testtools
 
 from oslo_log import _options
 from oslo_log import formatters
 from oslo_log import handlers
 from oslo_log import log
+from oslo_utils import units
 
 
 MIN_LOG_INI = b"""[loggers]
@@ -107,6 +112,7 @@ class CommonLoggerTestsMixIn(object):
                                                   '%(message)s')
         self.log = None
         log._setup_logging_from_conf(self.config_fixture.conf, 'test', 'test')
+        self.log_handlers = log.getLogger(None).logger.handlers
 
     def test_handlers_have_context_formatter(self):
         formatters_list = []
@@ -140,6 +146,77 @@ class CommonLoggerTestsMixIn(object):
                      'info', 'debug', 'log'):
             self.assertRaises(AttributeError, getattr, log, func)
 
+    @mock.patch('platform.system', return_value='Linux')
+    def test_eventlog_missing(self, platform_mock):
+        self.config(use_eventlog=True)
+        self.assertRaises(RuntimeError,
+                          log._setup_logging_from_conf,
+                          self.CONF,
+                          'test',
+                          'test')
+
+    @mock.patch('platform.system', return_value='Windows')
+    @mock.patch('logging.handlers.NTEventLogHandler')
+    @mock.patch('oslo_log.log.getLogger')
+    def test_eventlog(self, loggers_mock, handler_mock, platform_mock):
+        self.config(use_eventlog=True)
+        log._setup_logging_from_conf(self.CONF, 'test', 'test')
+        handler_mock.assert_called_once_with('test')
+        mock_logger = loggers_mock.return_value.logger
+        mock_logger.addHandler.assert_any_call(handler_mock.return_value)
+
+    @mock.patch('oslo_log.watchers.FastWatchedFileHandler')
+    @mock.patch('oslo_log.log._get_log_file_path', return_value='test.conf')
+    @mock.patch('platform.system', return_value='Linux')
+    def test_watchlog_on_linux(self, platfotm_mock, path_mock, handler_mock):
+        self.config(watch_log_file=True)
+        log._setup_logging_from_conf(self.CONF, 'test', 'test')
+        handler_mock.assert_called_once_with(path_mock.return_value)
+        self.assertEqual(self.log_handlers[0], handler_mock.return_value)
+
+    @mock.patch('logging.handlers.WatchedFileHandler')
+    @mock.patch('oslo_log.log._get_log_file_path', return_value='test.conf')
+    @mock.patch('platform.system', return_value='Windows')
+    def test_watchlog_on_windows(self, platform_mock, path_mock, handler_mock):
+        self.config(watch_log_file=True)
+        log._setup_logging_from_conf(self.CONF, 'test', 'test')
+        handler_mock.assert_called_once_with(path_mock.return_value)
+        self.assertEqual(self.log_handlers[0], handler_mock.return_value)
+
+    @mock.patch('logging.handlers.TimedRotatingFileHandler')
+    @mock.patch('oslo_log.log._get_log_file_path', return_value='test.conf')
+    def test_timed_rotate_log(self, path_mock, handler_mock):
+        rotation_type = 'interval'
+        when = 'weekday'
+        interval = 2
+        backup_count = 2
+        self.config(log_rotation_type=rotation_type,
+                    log_rotate_interval=interval,
+                    log_rotate_interval_type=when,
+                    max_logfile_count=backup_count)
+        log._setup_logging_from_conf(self.CONF, 'test', 'test')
+        handler_mock.assert_called_once_with(path_mock.return_value,
+                                             when='w2',
+                                             interval=interval,
+                                             backupCount=backup_count)
+        self.assertEqual(self.log_handlers[0], handler_mock.return_value)
+
+    @mock.patch('logging.handlers.RotatingFileHandler')
+    @mock.patch('oslo_log.log._get_log_file_path', return_value='test.conf')
+    def test_rotate_log(self, path_mock, handler_mock):
+        rotation_type = 'size'
+        max_logfile_size_mb = 100
+        maxBytes = max_logfile_size_mb * units.Mi
+        backup_count = 2
+        self.config(log_rotation_type=rotation_type,
+                    max_logfile_size_mb=max_logfile_size_mb,
+                    max_logfile_count=backup_count)
+        log._setup_logging_from_conf(self.CONF, 'test', 'test')
+        handler_mock.assert_called_once_with(path_mock.return_value,
+                                             maxBytes=maxBytes,
+                                             backupCount=backup_count)
+        self.assertEqual(self.log_handlers[0], handler_mock.return_value)
+
 
 class LoggerTestCase(CommonLoggerTestsMixIn, test_base.BaseTestCase):
     def setUp(self):
@@ -178,7 +255,7 @@ class LogTestBase(BaseTestCase):
         :param formatter: The formatter class to set on the handler.  Must be
             the class itself, not an instance.
         """
-        self.stream = six.StringIO()
+        self.stream = io.StringIO()
         if handler is None:
             handler = logging.StreamHandler
         self.handler = handler(self.stream)
@@ -291,17 +368,12 @@ class OSSysLogHandlerTestCase(BaseTestCase):
 
     def test_syslog(self):
         msg_unicode = u"Benoît Knecht & François Deppierraz login failure"
-        msg_utf8 = msg_unicode.encode('utf-8')
-
         handler = handlers.OSSysLogHandler()
         syslog.syslog = mock.Mock()
         handler.emit(
             logging.LogRecord("name", logging.INFO, "path", 123,
                               msg_unicode, None, None))
-        if six.PY2:
-            syslog.syslog.assert_called_once_with(syslog.LOG_INFO, msg_utf8)
-        else:
-            syslog.syslog.assert_called_once_with(syslog.LOG_INFO, msg_unicode)
+        syslog.syslog.assert_called_once_with(syslog.LOG_INFO, msg_unicode)
 
 
 class OSJournalHandlerTestCase(BaseTestCase):
@@ -322,35 +394,55 @@ class OSJournalHandlerTestCase(BaseTestCase):
         self.addCleanup(self.journal.stop)
         log.setup(self.CONF, 'testing')
 
+    @testtools.skipUnless(journal, "systemd journal binding is not available")
+    def test_handler(self):
+        handler = handlers.OSJournalHandler()
+        handler.emit(
+            logging.LogRecord("foo", logging.INFO,
+                              "path", 123, "hey!",
+                              None, None))
+        self.assertTrue(self.journal.send.called)
+
     def test_emit(self):
-        l = log.getLogger('nova-test.foo')
+        logger = log.getLogger('nova-test.foo')
         local_context = _fake_new_context()
-        l.info("Foo", context=local_context)
+        logger.info("Foo", context=local_context)
         self.assertEqual(
-            self.journal.send.call_args,
             mock.call(mock.ANY, CODE_FILE=mock.ANY, CODE_FUNC='test_emit',
                       CODE_LINE=mock.ANY, LOGGER_LEVEL='INFO',
                       LOGGER_NAME='nova-test.foo', PRIORITY=6,
+                      SYSLOG_FACILITY=syslog.LOG_USER,
                       SYSLOG_IDENTIFIER=mock.ANY,
                       REQUEST_ID=mock.ANY,
                       PROJECT_NAME='mytenant',
                       PROCESS_NAME='MainProcess',
                       THREAD_NAME='MainThread',
-                      USER_NAME='myuser'))
+                      USER_NAME='myuser'),
+            self.journal.send.call_args)
+        args, kwargs = self.journal.send.call_args
+        self.assertEqual(len(args), 1)
+        self.assertIsInstance(args[0], str)
+        self.assertIsInstance(kwargs['CODE_LINE'], int)
+        self.assertIsInstance(kwargs['PRIORITY'], int)
+        self.assertIsInstance(kwargs['SYSLOG_FACILITY'], int)
+        del kwargs['CODE_LINE'], kwargs['PRIORITY'], kwargs['SYSLOG_FACILITY']
+        for key, arg in kwargs.items():
+            self.assertIsInstance(key, str)
+            self.assertIsInstance(arg, (bytes, str))
 
     def test_emit_exception(self):
-        l = log.getLogger('nova-exception.foo')
+        logger = log.getLogger('nova-exception.foo')
         local_context = _fake_new_context()
         try:
             raise Exception("Some exception")
         except Exception:
-            l.exception("Foo", context=local_context)
+            logger.exception("Foo", context=local_context)
         self.assertEqual(
-            self.journal.send.call_args,
             mock.call(mock.ANY, CODE_FILE=mock.ANY,
                       CODE_FUNC='test_emit_exception',
                       CODE_LINE=mock.ANY, LOGGER_LEVEL='ERROR',
                       LOGGER_NAME='nova-exception.foo', PRIORITY=3,
+                      SYSLOG_FACILITY=syslog.LOG_USER,
                       SYSLOG_IDENTIFIER=mock.ANY,
                       REQUEST_ID=mock.ANY,
                       EXCEPTION_INFO=mock.ANY,
@@ -358,7 +450,18 @@ class OSJournalHandlerTestCase(BaseTestCase):
                       PROJECT_NAME='mytenant',
                       PROCESS_NAME='MainProcess',
                       THREAD_NAME='MainThread',
-                      USER_NAME='myuser'))
+                      USER_NAME='myuser'),
+            self.journal.send.call_args)
+        args, kwargs = self.journal.send.call_args
+        self.assertEqual(len(args), 1)
+        self.assertIsInstance(args[0], str)
+        self.assertIsInstance(kwargs['CODE_LINE'], int)
+        self.assertIsInstance(kwargs['PRIORITY'], int)
+        self.assertIsInstance(kwargs['SYSLOG_FACILITY'], int)
+        del kwargs['CODE_LINE'], kwargs['PRIORITY'], kwargs['SYSLOG_FACILITY']
+        for key, arg in kwargs.items():
+            self.assertIsInstance(key, str)
+            self.assertIsInstance(arg, (bytes, str))
 
 
 class LogLevelTestCase(BaseTestCase):
@@ -394,12 +497,12 @@ class LogLevelTestCase(BaseTestCase):
         self.assertEqual(log.TRACE, self.log_trace.logger.getEffectiveLevel())
 
     def test_child_log_has_level_of_parent_flag(self):
-        l = log.getLogger('nova-test.foo')
-        self.assertEqual(logging.INFO, l.logger.getEffectiveLevel())
+        logger = log.getLogger('nova-test.foo')
+        self.assertEqual(logging.INFO, logger.logger.getEffectiveLevel())
 
     def test_child_log_has_level_of_parent_flag_for_trace(self):
-        l = log.getLogger('nova-trace.foo')
-        self.assertEqual(log.TRACE, l.logger.getEffectiveLevel())
+        logger = log.getLogger('nova-trace.foo')
+        self.assertEqual(log.TRACE, logger.logger.getEffectiveLevel())
 
     def test_get_loggers(self):
         log._loggers['sentinel_log'] = mock.sentinel.sentinel_log
@@ -448,6 +551,10 @@ class JSONFormatterTestCase(LogTestBase):
         self.assertEqual(ctx.user_name, context['user_name'])
         self.assertEqual(ctx.project_name, context['project_name'])
         self.assertEqual('test-json', data['name'])
+        self.assertIn('request_id', context)
+        self.assertEqual(ctx.request_id, context['request_id'])
+        self.assertIn('global_request_id', context)
+        self.assertEqual(ctx.global_request_id, context['global_request_id'])
 
         self.assertEqual(test_msg % test_data, data['message'])
         self.assertEqual(test_msg, data['msg'])
@@ -515,15 +622,14 @@ class JSONFormatterTestCase(LogTestBase):
 
     def test_can_process_strings(self):
         expected = b'\\u2622'
-        if six.PY3:
-            # see ContextFormatterTestCase.test_can_process_strings
-            expected = '\\\\xe2\\\\x98\\\\xa2'
-        self.log.info(b'%s', u'\u2622'.encode('utf8'))
+        # see ContextFormatterTestCase.test_can_process_strings
+        expected = '\\\\xe2\\\\x98\\\\xa2'
+        self.log.info(b'%s', '\u2622'.encode('utf8'))
         self.assertIn(expected, self.stream.getvalue())
 
     def test_exception(self):
         ctxt = _fake_context()
-        ctxt.request_id = six.text_type('99')
+        ctxt.request_id = str('99')
         try:
             raise RuntimeError('test_exception')
         except RuntimeError:
@@ -535,7 +641,7 @@ class JSONFormatterTestCase(LogTestBase):
 
     def test_no_exception(self):
         ctxt = _fake_context()
-        ctxt.request_id = six.text_type('99')
+        ctxt.request_id = str('99')
         self.log.info('testing', context=ctxt)
         data = jsonutils.loads(self.stream.getvalue())
         self.assertIn('error_summary', data)
@@ -543,7 +649,7 @@ class JSONFormatterTestCase(LogTestBase):
 
     def test_exception_without_exc_info_passed(self):
         ctxt = _fake_context()
-        ctxt.request_id = six.text_type('99')
+        ctxt.request_id = str('99')
         try:
             raise RuntimeError('test_exception\ntraceback\nfrom\nremote error')
         except RuntimeError:
@@ -554,7 +660,7 @@ class JSONFormatterTestCase(LogTestBase):
 
     def test_exception_with_exc_info_passed(self):
         ctxt = _fake_context()
-        ctxt.request_id = six.text_type('99')
+        ctxt.request_id = str('99')
         try:
             raise RuntimeError('test_exception\ntraceback\nfrom\nremote error')
         except RuntimeError:
@@ -566,10 +672,6 @@ class JSONFormatterTestCase(LogTestBase):
                          data['error_summary'])
 
     def test_fallback(self):
-        if not formatters._HAVE_JSONUTILS_FALLBACK:
-            self.skipTest("need the fallback parameter of "
-                          "jsonutils.to_primitive() added in "
-                          "oslo_serialization 2.21.1")
 
         class MyObject(object):
             def __str__(self):
@@ -587,6 +689,22 @@ class JSONFormatterTestCase(LogTestBase):
         # convert it using repr() to prevent serialization error on logging.
         self.assertEqual(['repr'], data['args'])
 
+    def test_extra_args_filtered(self):
+        test_msg = 'This is a %(test)s line %%(unused)'
+        test_data = {'test': 'log', 'unused': 'removeme'}
+        self.log.debug(test_msg, test_data)
+
+        data = jsonutils.loads(self.stream.getvalue())
+        self.assertNotIn('unused', data['args'])
+
+    def test_entire_dict(self):
+        test_msg = 'This is a %s dict'
+        test_data = {'test': 'log', 'other': 'value'}
+        self.log.debug(test_msg, test_data)
+
+        data = jsonutils.loads(self.stream.getvalue())
+        self.assertEqual(test_data, data['args'])
+
 
 def get_fake_datetime(retval):
     class FakeDateTime(datetime.datetime):
@@ -634,6 +752,12 @@ class FluentFormatterTestCase(LogTestBase):
         self.assertEqual(local_context.user, context['user'])
         self.assertEqual('test-fluent', data['name'])
 
+        self.assertIn('request_id', context)
+        self.assertEqual(local_context.request_id, context['request_id'])
+        self.assertIn('global_request_id', context)
+        self.assertEqual(local_context.global_request_id,
+                         context['global_request_id'])
+
         self.assertEqual(test_msg % test_data, data['message'])
 
         self.assertEqual('test_log.py', data['filename'])
@@ -723,14 +847,14 @@ class ContextFormatterTestCase(LogTestBase):
 
     def test_message_logging_3rd_party_log_records(self):
         ctxt = _fake_context()
-        ctxt.request_id = six.text_type('99')
+        ctxt.request_id = str('99')
         sa_log = logging.getLogger('sqlalchemy.engine')
         sa_log.setLevel(logging.INFO)
-        message = self.trans_fixture.lazy('test ' + six.unichr(128))
+        message = self.trans_fixture.lazy('test ' + chr(128))
         sa_log.info(message)
 
         expected = ('HAS CONTEXT [%s]: %s\n' % (ctxt.request_id,
-                                                six.text_type(message)))
+                                                str(message)))
         self.assertEqual(expected, self.stream.getvalue())
 
     def test_debugging_log(self):
@@ -745,11 +869,11 @@ class ContextFormatterTestCase(LogTestBase):
         # the Message object, with a wrong encoding. This test case
         # tests that problem does not occur.
         ctxt = _fake_context()
-        ctxt.request_id = six.text_type('99')
-        message = self.trans_fixture.lazy('test ' + six.unichr(128))
+        ctxt.request_id = str('99')
+        message = self.trans_fixture.lazy('test ' + chr(128))
         self.log.info(message, context=ctxt)
         expected = "HAS CONTEXT [%s]: %s\n" % (ctxt.request_id,
-                                               six.text_type(message))
+                                               str(message))
         self.assertEqual(expected, self.stream.getvalue())
 
     def test_exception_logging(self):
@@ -757,8 +881,8 @@ class ContextFormatterTestCase(LogTestBase):
         # does not appear in the format string, ensure that it is
         # appended to the end of the log lines.
         ctxt = _fake_context()
-        ctxt.request_id = six.text_type('99')
-        message = self.trans_fixture.lazy('test ' + six.unichr(128))
+        ctxt.request_id = str('99')
+        message = self.trans_fixture.lazy('test ' + chr(128))
         try:
             raise RuntimeError('test_exception_logging')
         except RuntimeError:
@@ -770,8 +894,8 @@ class ContextFormatterTestCase(LogTestBase):
         # NOTE(dhellmann): Several of the built-in exception types
         # should not be automatically added to the log output.
         ctxt = _fake_context()
-        ctxt.request_id = six.text_type('99')
-        message = self.trans_fixture.lazy('test ' + six.unichr(128))
+        ctxt.request_id = str('99')
+        message = self.trans_fixture.lazy('test ' + chr(128))
         ignored_exceptions = [
             ValueError, TypeError, KeyError, AttributeError, ImportError
         ]
@@ -789,8 +913,8 @@ class ContextFormatterTestCase(LogTestBase):
         # that position in the output.
         self.config(logging_context_format_string="A %(error_summary)s B")
         ctxt = _fake_context()
-        ctxt.request_id = six.text_type('99')
-        message = self.trans_fixture.lazy('test ' + six.unichr(128))
+        ctxt.request_id = str('99')
+        message = self.trans_fixture.lazy('test ' + chr(128))
         try:
             raise RuntimeError('test_exception_logging')
         except RuntimeError:
@@ -804,32 +928,32 @@ class ContextFormatterTestCase(LogTestBase):
         # inserted.
         self.config(logging_context_format_string="%(error_summary)s")
         ctxt = _fake_context()
-        ctxt.request_id = six.text_type('99')
-        message = self.trans_fixture.lazy('test ' + six.unichr(128))
+        ctxt.request_id = str('99')
+        message = self.trans_fixture.lazy('test ' + chr(128))
         self.log.info(message, context=ctxt)
         expected = '-\n'
         self.assertTrue(self.stream.getvalue().startswith(expected))
 
     def test_unicode_conversion_in_adapter(self):
         ctxt = _fake_context()
-        ctxt.request_id = six.text_type('99')
+        ctxt.request_id = str('99')
         message = "Exception is (%s)"
-        ex = Exception(self.trans_fixture.lazy('test' + six.unichr(128)))
+        ex = Exception(self.trans_fixture.lazy('test' + chr(128)))
         self.log.debug(message, ex, context=ctxt)
-        message = six.text_type(message) % ex
+        message = str(message) % ex
         expected = "HAS CONTEXT [%s]: %s --DBG\n" % (ctxt.request_id,
                                                      message)
         self.assertEqual(expected, self.stream.getvalue())
 
     def test_unicode_conversion_in_formatter(self):
         ctxt = _fake_context()
-        ctxt.request_id = six.text_type('99')
+        ctxt.request_id = str('99')
         no_adapt_log = logging.getLogger('no_adapt')
         no_adapt_log.setLevel(logging.INFO)
         message = "Exception is (%s)"
-        ex = Exception(self.trans_fixture.lazy('test' + six.unichr(128)))
+        ex = Exception(self.trans_fixture.lazy('test' + chr(128)))
         no_adapt_log.info(message, ex)
-        message = six.text_type(message) % ex
+        message = str(message) % ex
         expected = "HAS CONTEXT [%s]: %s\n" % (ctxt.request_id,
                                                message)
         self.assertEqual(expected, self.stream.getvalue())
@@ -840,13 +964,13 @@ class ContextFormatterTestCase(LogTestBase):
                                                   "%(user_identity)s]: "
                                                   "%(message)s")
         ctxt = _fake_context()
-        ctxt.request_id = u'99'
+        ctxt.request_id = '99'
         message = 'test'
         self.log.info(message, context=ctxt)
         expected = ("HAS CONTEXT [%s %s %s %s %s %s]: %s\n" %
                     (ctxt.request_id, ctxt.user, ctxt.tenant, ctxt.domain,
                      ctxt.user_domain, ctxt.project_domain,
-                     six.text_type(message)))
+                     str(message)))
         self.assertEqual(expected, self.stream.getvalue())
 
     def test_user_identity_logging_set_format(self):
@@ -857,12 +981,12 @@ class ContextFormatterTestCase(LogTestBase):
                     logging_user_identity_format="%(user)s "
                                                  "%(tenant)s")
         ctxt = _fake_context()
-        ctxt.request_id = u'99'
+        ctxt.request_id = '99'
         message = 'test'
         self.log.info(message, context=ctxt)
         expected = ("HAS CONTEXT [%s %s %s]: %s\n" %
                     (ctxt.request_id, ctxt.user, ctxt.tenant,
-                     six.text_type(message)))
+                     str(message)))
         self.assertEqual(expected, self.stream.getvalue())
 
     @mock.patch("datetime.datetime",
@@ -896,12 +1020,11 @@ class ContextFormatterTestCase(LogTestBase):
 
     def test_can_process_strings(self):
         expected = b'\xe2\x98\xa2'
-        if six.PY3:
-            # in PY3 logging format string should be unicode string
-            # or it will fail and inserting byte string in unicode string
-            # causes such formatting
-            expected = '\\xe2\\x98\\xa2'
-        self.log.info(b'%s', u'\u2622'.encode('utf8'))
+        # logging format string should be unicode string
+        # or it will fail and inserting byte string in unicode string
+        # causes such formatting
+        expected = '\\xe2\\x98\\xa2'
+        self.log.info(b'%s', '\u2622'.encode('utf8'))
         self.assertIn(expected, self.stream.getvalue())
 
     def test_dict_args_with_unicode(self):
@@ -988,7 +1111,7 @@ class FancyRecordTestCase(LogTestBase):
         # and goes to stderr. Suggests on a better way to do this are
         # welcomed.
         error = sys.stderr
-        sys.stderr = six.StringIO()
+        sys.stderr = io.StringIO()
 
         self.colorlog.info("foo")
         self.assertNotEqual(-1,
@@ -1030,7 +1153,7 @@ class FancyRecordTestCase(LogTestBase):
         fake_resource = {'name': resource}
         message = 'info'
         self.colorlog.info(message, context=ctxt, resource=fake_resource)
-        expected = ('%s [%s]: [%s] %s\n' %
+        expected = ('%s [%s]: [%s] %s\033[00m\n' %
                     (color, ctxt.request_id, resource, message))
         self.assertEqual(expected, self.stream.getvalue())
 
@@ -1043,7 +1166,7 @@ class FancyRecordTestCase(LogTestBase):
                          'id': resource_id}
         message = 'info'
         self.colorlog.info(message, context=ctxt, resource=fake_resource)
-        expected = ('%s [%s]: [%s-%s] %s\n' %
+        expected = ('%s [%s]: [%s-%s] %s\033[00m\n' %
                     (color, ctxt.request_id, type, resource_id, message))
         self.assertEqual(expected, self.stream.getvalue())
 
@@ -1345,7 +1468,7 @@ class MutateTestCase(BaseTestCase):
         mock_fileConfig.reset_mock()
 
         # No thread sync going on here, just ensure the mtimes are different
-        time.sleep(0.1)
+        time.sleep(1)
         os.utime(logini, None)
         self.CONF.mutate_config_files()
 
@@ -1495,7 +1618,7 @@ keys=
         root = logging.getLogger()
         self.assertEqual(1, len(root.handlers))
         handler = root.handlers[0]
-        handler.stream = six.StringIO()
+        handler.stream = io.StringIO()
         return handler.stream
 
     def test_remove_handler(self):
@@ -1521,7 +1644,7 @@ keys=
                  'loggers': {'a.a': fake_logger}}
         conf2 = {'root': {'handlers': 'fake'},
                  'handlers': {'fake': fake_handler}}
-        stream = six.StringIO()
+        stream = io.StringIO()
         with self.mutate_conf(conf1, conf2) as (loginis, confs):
             stream = self.set_root_stream()
             log = logging.getLogger("a.a")
@@ -1540,7 +1663,7 @@ class LogConfigOptsTestCase(BaseTestCase):
         super(LogConfigOptsTestCase, self).setUp()
 
     def test_print_help(self):
-        f = six.StringIO()
+        f = io.StringIO()
         self.CONF([])
         self.CONF.print_help(file=f)
         for option in ['debug', 'log-config', 'watch-log-file']:
@@ -1692,6 +1815,20 @@ class LogConfigTestCase(BaseTestCase):
                                            disable_existing_loggers=False)
 
 
+class SavingAdapter(log.KeywordArgumentAdapter):
+
+    def __init__(self, *args, **kwds):
+        super(log.KeywordArgumentAdapter, self).__init__(*args, **kwds)
+        self.results = []
+
+    def process(self, msg, kwargs):
+        # Run the real adapter and save the inputs and outputs
+        # before returning them so the test can examine both.
+        results = super(SavingAdapter, self).process(msg, kwargs)
+        self.results.append((msg, kwargs, results))
+        return results
+
+
 class KeywordArgumentAdapterTestCase(BaseTestCase):
 
     def setUp(self):
@@ -1744,91 +1881,83 @@ class KeywordArgumentAdapterTestCase(BaseTestCase):
             kwargs)
 
     def test_pass_args_to_log(self):
-        a = log.KeywordArgumentAdapter(self.mock_log, {})
+        a = SavingAdapter(self.mock_log, {})
+
         message = 'message'
         exc_message = 'exception'
-        key = 'name'
         val = 'value'
         a.log(logging.DEBUG, message, name=val, exc_info=exc_message)
-        if six.PY3:
-            self.mock_log._log.assert_called_once_with(
-                logging.DEBUG,
-                message,
-                (),
-                extra={key: val, 'extra_keys': [key]},
-                exc_info=exc_message
-            )
-        else:
-            self.mock_log.log.assert_called_once_with(
-                logging.DEBUG,
-                message,
-                extra={key: val, 'extra_keys': [key]},
-                exc_info=exc_message
-            )
+
+        expected = {
+            'exc_info': exc_message,
+            'extra': {'name': val, 'extra_keys': ['name']},
+        }
+
+        actual = a.results[0]
+        self.assertEqual(message, actual[0])
+        self.assertEqual(expected, actual[1])
+        results = actual[2]
+        self.assertEqual(message, results[0])
+        self.assertEqual(expected, results[1])
 
     def test_pass_args_via_debug(self):
-        a = log.KeywordArgumentAdapter(self.mock_log, {})
+
+        a = SavingAdapter(self.mock_log, {})
         message = 'message'
         exc_message = 'exception'
-        key = 'name'
         val = 'value'
         a.debug(message, name=val, exc_info=exc_message)
-        # The adapter implementation for debug() is different for
-        # python 3, so we expect a different method to be called
-        # internally.
-        if six.PY3:
-            self.mock_log._log.assert_called_once_with(
-                logging.DEBUG,
-                message,
-                (),
-                extra={key: val, 'extra_keys': [key]},
-                exc_info=exc_message
-            )
-        else:
-            self.mock_log.debug.assert_called_once_with(
-                message,
-                extra={key: val, 'extra_keys': [key]},
-                exc_info=exc_message
-            )
+
+        expected = {
+            'exc_info': exc_message,
+            'extra': {'name': val, 'extra_keys': ['name']},
+        }
+
+        actual = a.results[0]
+        self.assertEqual(message, actual[0])
+        self.assertEqual(expected, actual[1])
+        results = actual[2]
+        self.assertEqual(message, results[0])
+        self.assertEqual(expected, results[1])
 
 
 class UnicodeConversionTestCase(BaseTestCase):
 
-    _MSG = u'Message with unicode char \ua000 in the middle'
+    _MSG = 'Message with unicode char \ua000 in the middle'
 
     def test_ascii_to_unicode(self):
         msg = self._MSG
         enc_msg = msg.encode('utf-8')
         result = formatters._ensure_unicode(enc_msg)
         self.assertEqual(msg, result)
-        self.assertIsInstance(result, six.text_type)
+        self.assertIsInstance(result, str)
 
     def test_unicode_to_unicode(self):
         msg = self._MSG
         result = formatters._ensure_unicode(msg)
         self.assertEqual(msg, result)
-        self.assertIsInstance(result, six.text_type)
+        self.assertIsInstance(result, str)
 
     def test_exception_to_unicode(self):
         msg = self._MSG
         exc = Exception(msg)
         result = formatters._ensure_unicode(exc)
         self.assertEqual(msg, result)
-        self.assertIsInstance(result, six.text_type)
+        self.assertIsInstance(result, str)
 
 
 class LoggerNameTestCase(LoggerTestCase):
 
     def test_oslo_dot(self):
         logger_name = 'oslo.subname'
-        l = log.getLogger(logger_name)
-        self.assertEqual(logger_name, l.logger.name)
+        logger = log.getLogger(logger_name)
+        self.assertEqual(logger_name, logger.logger.name)
 
     def test_oslo_underscore(self):
         logger_name = 'oslo_subname'
         expected = logger_name.replace('_', '.')
-        l = log.getLogger(logger_name)
-        self.assertEqual(expected, l.logger.name)
+        logger = log.getLogger(logger_name)
+        self.assertEqual(expected, logger.logger.name)
 
 
 class IsDebugEnabledTestCase(test_base.BaseTestCase):
